# -*- coding: utf-8 -*-
from zope.interface.common import mapping
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class ICollectiveSAConnectLayer(IDefaultBrowserLayer):
    """Browserlayer
    """


class ISQLAlchemyConnectionStrings(mapping.IMapping):
    """A collection of SQLAlchemy connection strings"""



# -*- coding: utf-8 -*-
from collective.saconnect import MessageFactory as _
from collective.saconnect.interfaces import ISQLAlchemyConnectionStrings
from plone.app.z3cform.layout import wrap_form
from plone.z3cform.crud import crud
from Products.CMFCore.interfaces import ISiteRoot
from zope import schema
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface


class IConnectionLine(Interface):
    """One row in the form
    """
    connname = schema.ASCIILine(
        title=_(u'label_connection_name', default=u'Connection name'),
        description=_(u'description_connection_name', default=u''),
        required=True,
    )
    connstring = schema.ASCIILine(
        title=_(u"label_connection_string",
                default=u"SQLAlchemy connection string"),
        description=_(u"description_connection_string",
                      default=u"driver://user:password@hostname/database"),
        required=True,
        default="sqlite:///"
    )


@implementer(IConnectionLine)
class ConnectionLine(object):

    def __init__(self, name, string, form):
        self._connname = name
        self._connstring = string
        self._form = form

    @property
    def connname(self):
        return self._connname

    @connname.setter
    def connname(self, name):
        name = name.strip()
        if name == self._connname:
            return

        storage = self._form.storage
        del storage[self._connname]
        storage[name] = self.connstring
        self._connname = name

    @property
    def connstring(self):
        return self._connstring

    @connstring.setter
    def connstring(self, string):
        string = string.strip()
        if string == self._connstring:
            return

        storage = self._form.storage
        storage[self.connname] = self._connstring = string


class SQLAlchemyConnectionsForm(crud.CrudForm):
    label = _(u'label_saconnectform', u'Manage SQLAlchemy connection strings')
    update_schema = IConnectionLine

    def __init__(self, context, request):
        crud.CrudForm.__init__(self, context, request)
        self.storage = ISQLAlchemyConnectionStrings(
            getUtility(ISiteRoot)
        )

    def get_items(self):
        names = self.storage.keys()
        names.sort()
        return [(name, ConnectionLine(name, self.storage[name], self))
                for name in names]

    def add(self, data):
        name = data['connname'].strip()
        if str(name) in self.storage:
            msg = _(u'error_name_already_registered',
                    u'The connection name is already registered')
            msg = translate(msg, self.request)
            raise schema.ValidationError(msg)

        self.storage[name] = data['connstring'].strip()
        return ConnectionLine(name, data['connstring'].strip(), self)

    def remove(self, (id, item)):
        del self.storage[item.connname]

SQLAlchemyConnectionsView = wrap_form(SQLAlchemyConnectionsForm)



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.saconnect.testing import COLLECTIVE_SACONNECT_INTEGRATION_TESTING  # noqa
from Products.CMFCore.utils import getToolByName

import unittest


class TestSetup(unittest.TestCase):
    """Test that collective.saconnect is properly installed."""

    layer = COLLECTIVE_SACONNECT_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = getToolByName(self.portal, 'portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.saconnect is installed with
           portal_quickinstaller.
        """
        self.assertTrue(
            self.installer.isProductInstalled(
                'collective.saconnect'
            )
        )

    def test_uninstall(self):
        """Test if collective.saconnect is cleanly uninstalled."""
        self.installer.uninstallProducts(['collective.saconnect'])
        self.assertFalse(
            self.installer.isProductInstalled(
                'collective.saconnect'
            )
        )
        # self.assertNotIn('authomatic', self.portal.acl_users)

    def test_browserlayer(self):
        """Test that IPasPluginsAuthomaticLayer is registered."""
        from collective.saconnect.interfaces import ICollectiveSAConnectLayer  # noqa
        from plone.browserlayer import utils
        self.assertTrue(
            ICollectiveSAConnectLayer in utils.registered_layers()
        )



# -*- coding: utf-8 -*-
# optional z3c.saconfig support
from collective.saconnect.interfaces import ISQLAlchemyConnectionStrings
from persistent import Persistent
from Products.CMFCore.interfaces import ISiteRoot
from z3c.saconfig.interfaces import IEngineFactory
from z3c.saconfig.interfaces import ISiteScopedSession
from z3c.saconfig.utility import EngineFactory
from z3c.saconfig.utility import SESSION_DEFAULTS
from z3c.saconfig.utility import SiteScopedSession
from zope.component import getSiteManager
from zope.component import getUtility
from zope.interface import implementer


class ISiteScopedSessionEngineFactory(ISiteScopedSession, IEngineFactory):
    """All in one z3c.saconfig utility"""


@implementer(ISiteScopedSessionEngineFactory)
class SiteScopedSessionEngineFactory(
    SiteScopedSession,
    EngineFactory,
    Persistent
):
    """An all in one z3c.saconfig local persistent utility

    TODO: add configuration options
    """

    def __init__(self, name):
        self.name = name

    @property
    def _key(self):
        siteroot = getUtility(ISiteRoot)
        return "collective.saconnect:%s,%s" % (self.name, siteroot._p_oid)

    def configuration(self):
        siteroot = getUtility(ISiteRoot)
        connections = ISQLAlchemyConnectionStrings(siteroot)
        url = connections[self.name]
        return [url], dict(convert_unicode=True)

    # SiteScopedSession

    @property
    def engine(self):
        return self.name

    @property
    def kw(self):
        return SESSION_DEFAULTS

    def siteScopeFunc(self):
        return id(getSiteManager())


def syncUtility(connections, event):
    """Subscriber
    """
    sm = getSiteManager()
    for key in event.descriptions:
        factory = sm.queryUtility(ISiteScopedSessionEngineFactory, name=key)
        if key in connections.keys():
            if factory is not None:
                factory.reset()  # modify
                continue
            factory = SiteScopedSessionEngineFactory(key)
            sm.registerUtility(  # add
                factory,
                name=key,
                provided=ISiteScopedSessionEngineFactory,
            )
            continue
        if factory is not None:
            factory.reset()
            sm.unregisterUtility(  # delete
                factory,
                name=key,
            )



# -*- coding: utf-8 -*-
from collective.saconnect.interfaces import ISQLAlchemyConnectionStrings
from Products.GenericSetup.interfaces import IBody
from Products.GenericSetup.interfaces import ISetupEnviron
from Products.GenericSetup.utils import XMLAdapterBase
from zope import component

FILENAME = 'saconnections.xml'


class SQLAlchemyConnectionStringsXMLAdapter(XMLAdapterBase):
    """XML im- and exporter for SQLAlchemy Connection strings"""
    component.adapts(ISQLAlchemyConnectionStrings, ISetupEnviron)

    _LOGGER_ID = 'collective.saconnect'

    def _exportNode(self):
        node = self._doc.createElement('connections')
        names = self.context.keys()
        names.sort()

        for name in names:
            child = self._doc.createElement('connection')
            child.setAttribute('name', name)
            child.setAttribute('string', self.context[name])
            node.appendChild(child)

        self._logger.info('Exported: %s' % ', '.join(names))
        return node

    def _importNode(self, node):
        if self.environ.shouldPurge():
            self.context.clear()
            self._logger.info('Connection strings purged.')

        imported = []
        for child in node.childNodes:
            if child.nodeName == 'connection':
                name = child.getAttribute('name').encode('ascii')
                if child.hasAttribute('remove'):
                    if name in self.context:
                        del self.context[name]
                    continue

                conn = child.getAttribute('string').encode('ascii')
                self.context[name] = conn
                imported.append(name)
        if imported:
            self._logger.info('Imported: %s' % ', '.join(imported))


def importConnections(context):
    data = context.readDataFile(FILENAME)
    if not data:
        return

    storage = ISQLAlchemyConnectionStrings(context.getSite())
    importer = component.queryMultiAdapter((storage, context), IBody)
    if importer:
        importer.body = data


def exportConnections(context):
    storage = ISQLAlchemyConnectionStrings(context.getSite())
    exporter = component.queryMultiAdapter((storage, context), IBody)
    if exporter:
        data = exporter.body
        if data is not None:
            context.writeDataFile(FILENAME, data, exporter.mime_type)



# -*- coding: utf-8 -*-
from collective.saconnect.interfaces import ISQLAlchemyConnectionStrings
from collective.saconnect.testing import COLLECTIVE_SACONNECT_INTEGRATION_TESTING  # noqa
from Products.GenericSetup.testing import BodyAdapterTestCase
from UserDict import UserDict
from zope.component import provideAdapter
from zope.interface import directlyProvides

import unittest


TESTBODY = '''\
<?xml version="1.0"?>
<connections>
 <connection name="bar" string="eggs"/>
 <connection name="foo" string="spam"/>
</connections>
'''


class TestGS(unittest.TestCase, BodyAdapterTestCase):
    """Test that collective.saconnect is properly installed."""

    layer = COLLECTIVE_SACONNECT_INTEGRATION_TESTING

    _BODY = TESTBODY

    def setUp(self):
        self._obj = UserDict()
        directlyProvides(self._obj, ISQLAlchemyConnectionStrings)
        provideAdapter(self._getTargetClass())

    def _getTargetClass(self):
        from collective.saconnect.genericsetup import SQLAlchemyConnectionStringsXMLAdapter  # noqa
        return SQLAlchemyConnectionStringsXMLAdapter

    def _populate(self, obj):
        obj['foo'] = 'spam'
        obj['bar'] = 'eggs'

    def _verifyImport(self, obj):
        self.assertEqual(obj, dict(foo='spam', bar='eggs'))



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup
import os

version = '2.0'

long_desc = open("README.rst").read() + "\n"
long_desc += open(os.path.join("CHANGES.rst")).read()

setup(
    name='collective.saconnect',
    version=version,
    description="Plone Control Panel for SQL Alchemy connection strings",
    long_description=long_desc,
    classifiers=[
        "Programming Language :: Python",
    ],
    keywords='sqlalchemy plone zope2',
    author='Plone Community (started by Jarn AS)',
    author_email='info@jarn.com',
    url='http://pypi.python.org/pypi/collective.saconnect',
    license='GPL',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['collective'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Plone',
        'plone.app.z3cform',
        'setuptools',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'z3c.saconfig',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from interfaces import ISQLAlchemyConnectionStrings
from persistent.mapping import PersistentMapping
from UserDict import DictMixin
from zope.annotation.interfaces import IAnnotatable
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.interface import implementer
from zope.lifecycleevent import modified

ANNKEY = 'collective.saconnect.storage'


@implementer(ISQLAlchemyConnectionStrings)
@adapter(IAnnotatable)
class SQLAlchemyConnectionStrings(DictMixin):

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(self.context)
        self._dict = annotations.setdefault(ANNKEY, PersistentMapping())

    def __getitem__(self, key):
        return self._dict[key]

    def __setitem__(self, key, value):
        if not isinstance(key, str):
            raise ValueError('Only plain ascii keys are accepted')
        if not isinstance(value, str):
            raise ValueError('Only plain ascii values are accepted')
        self._dict[key] = value
        modified(self, key)

    def __delitem__(self, key):
        del self._dict[key]
        modified(self, key)

    def keys(self):
        return self._dict.keys()



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer

import collective.saconnect


class CollectiveSAConnectLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML(package=collective.saconnect)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.saconnect:default')


COLLECTIVE_SACONNECT_FIXTURE = CollectiveSAConnectLayer()


COLLECTIVE_SACONNECT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_SACONNECT_FIXTURE,),
    name='CollectiveSAConnectLayer:IntegrationTesting'
)


COLLECTIVE_SACONNECT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_SACONNECT_FIXTURE,),
    name='CollectiveSAConnectLayer:FunctionalTesting'
)
